# Copyright 2020 The AutoKeras Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

BERT_CHECKPOINT_PATH = "https://github.com/keras-team/autokeras/releases/download/1.0.13/bert_checkpoint.tar.gz"  # noqa: E501
BERT_VOCAB_PATH = "https://github.com/keras-team/autokeras/releases/download/1.0.13/bert_vocab.txt"  # noqa: E501
